@@ -19,6 +19,7 @@ import sys
 
 from openstack import config as os_config
 
+from metalsmith import _config
 from metalsmith import _format
 from metalsmith import _provisioner
 from metalsmith import _utils
@@ -51,12 +52,16 @@ def _do_deploy(api, args, formatter):
     if args.hostname and not _utils.is_hostname_safe(args.hostname):
         raise RuntimeError("%s cannot be used as a hostname" % args.hostname)
 
+    config = _config.InstanceConfig(ssh_keys=ssh_keys)
+    if args.user_name:
+        config.add_user(args.user_name, sudo=args.passwordless_sudo)
+
     node = api.reserve_node(args.resource_class, capabilities=capabilities)
     instance = api.provision_node(node,
-                                  image_ref=args.image,
+                                  image=args.image,
                                   nics=args.nics,
                                   root_disk_size=args.root_disk_size,
-                                  ssh_keys=ssh_keys,
+                                  config=config,
                                   hostname=args.hostname,
                                   netboot=args.netboot,
                                   wait=wait)
@@ -119,7 +124,11 @@ def _parse_args(args, config):
     deploy.add_argument('--ssh-public-key', help='SSH public key to load')
     deploy.add_argument('--hostname', help='Host name to use, defaults to '
                         'Node\'s name or UUID')
-    deploy.add_argument('resource_class', help='node resource class to deploy')
+    deploy.add_argument('--resource-class', required=True,
+                        help='node resource class to deploy')
+    deploy.add_argument('--user-name', help='Name of the admin user to create')
+    deploy.add_argument('--passwordless-sudo', action='store_true',
+                        help='allow password-less sudo for the user')
 
     undeploy = subparsers.add_parser('undeploy')
     undeploy.set_defaults(func=_do_undeploy)
