@@ -13,13 +13,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import collections
 import logging
 import random
 import sys
+import time
 
 import six
 
+from metalsmith import _config
+from metalsmith import _instance
 from metalsmith import _os_api
 from metalsmith import _scheduler
 from metalsmith import _utils
@@ -30,115 +32,6 @@ LOG = logging.getLogger(__name__)
 
 _CREATED_PORTS = 'metalsmith_created_ports'
 _ATTACHED_PORTS = 'metalsmith_attached_ports'
-# NOTE(dtantsur): include available since there is a period of time between
-# claiming the instance and starting the actual provisioning via ironic.
-_DEPLOYING_STATES = frozenset(['available', 'deploying', 'wait call-back',
-                               'deploy complete'])
-_ACTIVE_STATES = frozenset(['active'])
-_ERROR_STATE = frozenset(['error', 'deploy failed'])
-
-_HEALTHY_STATES = frozenset(['deploying', 'active'])
-
-
-class Instance(object):
-    """Instance status in metalsmith."""
-
-    def __init__(self, api, node):
-        self._api = api
-        self._uuid = node.uuid
-        self._node = node
-
-    @property
-    def hostname(self):
-        """Node's hostname."""
-        return self._node.instance_info.get(_os_api.HOSTNAME_FIELD)
-
-    def ip_addresses(self):
-        """Returns IP addresses for this instance.
-
-        :return: dict mapping network name or ID to a list of IP addresses.
-        """
-        result = {}
-        for nic in self.nics():
-            net = getattr(nic.network, 'name', None) or nic.network.id
-            result.setdefault(net, []).extend(
-                ip['ip_address'] for ip in nic.fixed_ips
-                if ip.get('ip_address')
-            )
-        return result
-
-    @property
-    def is_deployed(self):
-        """Whether the node is deployed."""
-        return self._node.provision_state in _ACTIVE_STATES
-
-    @property
-    def is_healthy(self):
-        """Whether the node is not at fault or maintenance."""
-        return self.state in _HEALTHY_STATES and not self._node.maintenance
-
-    def nics(self):
-        """List NICs for this instance.
-
-        :return: List of `Port` objects with additional ``network`` fields
-            with full representations of their networks.
-        """
-        result = []
-        vifs = self._api.list_node_attached_ports(self.node)
-        for vif in vifs:
-            port = self._api.get_port(vif.id)
-            port.network = self._api.get_network(port.network_id)
-            result.append(port)
-        return result
-
-    @property
-    def node(self):
-        """Underlying `Node` object."""
-        return self._node
-
-    @property
-    def state(self):
-        """Instance state.
-
-        ``deploying``
-            deployment is in progress
-        ``active``
-            node is provisioned
-        ``maintenance``
-            node is provisioned but is in maintenance mode
-        ``error``
-            node has a failure
-        ``unknown``
-            node in unexpected state (maybe unprovisioned or modified by
-            a third party)
-        """
-        prov_state = self._node.provision_state
-        if prov_state in _DEPLOYING_STATES:
-            return 'deploying'
-        elif prov_state in _ERROR_STATE:
-            return 'error'
-        elif prov_state in _ACTIVE_STATES:
-            if self._node.maintenance:
-                return 'maintenance'
-            else:
-                return 'active'
-        else:
-            return 'unknown'
-
-    def to_dict(self):
-        """Convert instance to a dict."""
-        return {
-            'hostname': self.hostname,
-            'ip_addresses': self.ip_addresses(),
-            'node': self._node.to_dict(),
-            'state': self.state,
-            'uuid': self._uuid,
-        }
-
-    @property
-    def uuid(self):
-        """Instance UUID (the same as `Node` UUID for metalsmith)."""
-        return self._uuid
 
 
 class Provisioner(object):
@@ -252,8 +145,8 @@ class Provisioner(object):
 
         return hostname
 
-    def provision_node(self, node, image_ref, nics=None, root_disk_size=None,
-                       ssh_keys=None, hostname=None, netboot=False, wait=None):
+    def provision_node(self, node, image, nics=None, root_disk_size=None,
+                       config=None, hostname=None, netboot=False, wait=None):
         """Provision the node with the given image.
 
         Example::
@@ -267,15 +160,15 @@ class Provisioner(object):
         :param node: Node object, UUID or name. Will be reserved first, if
             not reserved already. Must be in the "available" state with
             maintenance mode off.
-        :param image_ref: Image name or UUID to provision.
+        :param image: Image name or UUID to provision.
         :param nics: List of virtual NICs to attach to physical ports.
             Each item is a dict with a key describing the type of the NIC:
             either a port (``{"port": "<port name or ID>"}``) or a network
             to create a port on (``{"network": "<network name or ID>"}``).
         :param root_disk_size: The size of the root partition. By default
             the value of the local_gb property is used.
-        :param ssh_keys: list of public parts of the SSH keys to upload
-            to the nodes.
+        :param config: :py:class:`metalsmith.InstanceConfig` object with
+            the configuration to pass to the instance.
         :param hostname: Hostname to assign to the instance. Defaults to the
             node's name or UUID.
         :param netboot: Whether to use networking boot for final instances.
@@ -286,6 +179,8 @@ class Provisioner(object):
             is already finished.
         :raises: :py:class:`metalsmith.exceptions.Error`
         """
+        if config is None:
+            config = _config.InstanceConfig()
         node = self._check_node_for_deploy(node)
         created_ports = []
         attached_ports = []
@@ -295,11 +190,11 @@ class Provisioner(object):
             root_disk_size = _utils.get_root_disk(root_disk_size, node)
 
             try:
-                image = self._api.get_image_info(image_ref)
+                image = self._api.get_image(image)
             except Exception as exc:
                 raise exceptions.InvalidImage(
                     'Cannot find image %(image)s: %(error)s' %
-                    {'image': image_ref, 'error': exc})
+                    {'image': image, 'error': exc})
 
             LOG.debug('Image: %s', image)
 
@@ -334,7 +229,7 @@ class Provisioner(object):
 
             LOG.debug('Generating a configdrive for node %s',
                       _utils.log_node(node))
-            with _utils.config_drive_dir(node, ssh_keys, hostname) as cd:
+            with config.build_configdrive_directory(node, hostname) as cd:
                 self._api.node_action(node, 'active',
                                       configdrive=cd)
         except Exception:
@@ -343,9 +238,8 @@ class Provisioner(object):
             try:
                 LOG.error('Deploy attempt failed on node %s, cleaning up',
                           _utils.log_node(node))
-                self._delete_ports(node, created_ports, attached_ports)
-                LOG.debug('Releasing lock on node %s', _utils.log_node(node))
-                self._api.release_node(node)
+                self._clean_up(node, created_ports=created_ports,
+                               attached_ports=attached_ports)
             except Exception:
                 LOG.exception('Clean up failed')
 
@@ -357,24 +251,20 @@ class Provisioner(object):
             LOG.debug('Waiting for node %(node)s to reach state active '
                       'with timeout %(timeout)s',
                       {'node': _utils.log_node(node), 'timeout': wait})
-            self._api.wait_for_node_state(node, 'active', timeout=wait)
+            node = self.wait_for_provisioning([node], timeout=wait)[0]
             LOG.info('Deploy succeeded on node %s', _utils.log_node(node))
+        else:
+            # Update the node to return it's latest state
+            node = self._api.get_node(node, refresh=True)
 
-        # Update the node to return it's latest state
-        node = self._api.get_node(node, refresh=True)
-        return Instance(self._api, node)
+        return _instance.Instance(self._api, node)
 
     def _get_nics(self, nics):
         """Validate and get the NICs."""
+        _utils.validate_nics(nics)
+
         result = []
-        if not isinstance(nics, collections.Sequence):
-            raise TypeError("NICs must be a list of dicts")
-
         for nic in nics:
-            if not isinstance(nic, collections.Mapping) or len(nic) != 1:
-                raise TypeError("Each NIC must be a dict with one item, "
-                                "got %s" % nic)
-
             nic_type, nic_id = next(iter(nic.items()))
             if nic_type == 'network':
                 try:
@@ -385,7 +275,7 @@ class Provisioner(object):
                         {'net': nic_id, 'error': exc})
                 else:
                     result.append((nic_type, network))
-            elif nic_type == 'port':
+            else:
                 try:
                     port = self._api.get_port(nic_id)
                 except Exception as exc:
@@ -394,9 +284,6 @@ class Provisioner(object):
                         {'port': nic_id, 'error': exc})
                 else:
                     result.append((nic_type, port))
-            else:
-                raise ValueError("Unexpected NIC type %s, supported values: "
-                                 "'port', 'network'" % nic_type)
 
         return result
 
@@ -460,6 +347,87 @@ class Provisioner(object):
             LOG.debug('Failed to clear node %(node)s extra: %(exc)s',
                       {'node': _utils.log_node(node), 'exc': exc})
 
+    def wait_for_provisioning(self, nodes, timeout=None, delay=15):
+        """Wait for nodes to be provisioned.
+
+        Loops until all nodes finish provisioning.
+
+        :param nodes: List of nodes (UUID, name, `Node` object or
+            :py:class:`metalsmith.Instance`).
+        :param timeout: How much time (in seconds) to wait for all nodes
+            to finish provisioning. If ``None`` (the default), wait forever
+            (more precisely, until the operation times out on server side).
+        :param delay: Delay (in seconds) between two provision state checks.
+        :return: List of updated nodes if all succeeded.
+        :raises: :py:class:`metalsmith.exceptions.DeploymentFailure`
+            if the deployment failed or timed out for any nodes.
+        """
+        return self._wait_for_state(nodes, 'active',
+                                    timeout=timeout, delay=delay)
+
+    def _wait_for_state(self, nodes, state, timeout, delay=15):
+        if timeout is not None and timeout <= 0:
+            raise ValueError("The timeout argument must be a positive int")
+        if delay < 0:
+            raise ValueError("The delay argument must be a non-negative int")
+
+        failed_nodes = []
+        finished_nodes = []
+
+        deadline = time.time() + timeout if timeout is not None else None
+        while timeout is None or time.time() < deadline:
+            remaining_nodes = []
+            for node in nodes:
+                node = self._api.get_node(node, refresh=True,
+                                          accept_hostname=True)
+                if node.provision_state == state:
+                    LOG.debug('Node %(node)s reached state %(state)s',
+                              {'node': _utils.log_node(node), 'state': state})
+                    finished_nodes.append(node)
+                elif (node.provision_state == 'error' or
+                      node.provision_state.endswith(' failed')):
+                    LOG.error('Node %(node)s failed deployment: %(error)s',
+                              {'node': _utils.log_node(node),
+                               'error': node.last_error})
+                    failed_nodes.append(node)
+                else:
+                    remaining_nodes.append(node)
+
+            if remaining_nodes:
+                nodes = remaining_nodes
+            else:
+                nodes = []
+                break
+
+            LOG.debug('Still waiting for the following nodes to reach state '
+                      '%(state)s: %(nodes)s',
+                      {'state': state,
+                       'nodes': ', '.join(_utils.log_node(n) for n in nodes)})
+            time.sleep(delay)
+
+        messages = []
+        if failed_nodes:
+            messages.append('the following nodes failed deployment: %s' %
+                            ', '.join('%s (%s)' % (_utils.log_node(node),
+                                                   node.last_error)
+                                      for node in failed_nodes))
+        if nodes:
+            messages.append('deployment timed out for nodes %s' %
+                            ', '.join(_utils.log_node(node) for node in nodes))
+
+        if messages:
+            raise exceptions.DeploymentFailure(
+                'Deployment failed: %s' % '; '.join(messages),
+                failed_nodes + nodes)
+        else:
+            LOG.debug('All nodes reached state %s', state)
+            return finished_nodes
+
+    def _clean_up(self, node, created_ports=None, attached_ports=None):
+        self._delete_ports(node, created_ports, attached_ports)
+        LOG.debug('Releasing lock on node %s', _utils.log_node(node))
+        self._api.release_node(node)
+
     def unprovision_node(self, node, wait=None):
         """Unprovision a previously provisioned node.
 
@@ -474,15 +442,13 @@ class Provisioner(object):
             LOG.warning("Dry run, not unprovisioning")
             return
 
-        self._delete_ports(node)
-        LOG.debug('Releasing lock on node %s', _utils.log_node(node))
-        self._api.release_node(node)
+        self._clean_up(node)
         self._api.node_action(node, 'deleted')
 
         LOG.info('Deleting started for node %s', _utils.log_node(node))
 
         if wait is not None:
-            self._api.wait_for_node_state(node, 'available', timeout=wait)
+            self._wait_for_state([node], 'available', timeout=wait)
             LOG.info('Node %s undeployed successfully', _utils.log_node(node))
 
         return self._api.get_node(node, refresh=True)
@@ -506,6 +472,9 @@ class Provisioner(object):
             order as ``instances``.
         """
         with self._api.cache_node_list_for_lookup():
-            return [Instance(self._api,
-                             self._api.get_node(inst, accept_hostname=True))
-                    for inst in instances]
+            return [
+                _instance.Instance(
+                    self._api,
+                    self._api.get_node(inst, accept_hostname=True))
+                for inst in instances
+            ]
