@@ -13,12 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import contextlib
-import json
-import os
+import collections
 import re
-import shutil
-import tempfile
 
 import six
 
@@ -46,31 +42,6 @@ def get_capabilities(node):
     return caps
 
 
-@contextlib.contextmanager
-def config_drive_dir(node, ssh_keys, hostname):
-    d = tempfile.mkdtemp()
-    try:
-        metadata = {'public_keys': ssh_keys,
-                    'uuid': node.uuid,
-                    'name': node.name,
-                    'hostname': hostname,
-                    'launch_index': 0,
-                    'availability_zone': '',
-                    'files': [],
-                    'meta': {}}
-        for version in ('2012-08-10', 'latest'):
-            subdir = os.path.join(d, 'openstack', version)
-            if not os.path.exists(subdir):
-                os.makedirs(subdir)
-
-            with open(os.path.join(subdir, 'meta_data.json'), 'w') as fp:
-                json.dump(metadata, fp)
-
-        yield d
-    finally:
-        shutil.rmtree(d)
-
-
 def get_root_disk(root_disk_size, node):
     """Validate and calculate the root disk size."""
     if root_disk_size is not None:
@@ -121,3 +92,23 @@ def is_hostname_safe(hostname):
         return False
 
     return _HOSTNAME_RE.match(hostname) is not None
+
+
+def validate_nics(nics):
+    """Validate NICs."""
+    if not isinstance(nics, collections.Sequence):
+        raise TypeError("NICs must be a list of dicts")
+
+    unknown_nic_types = set()
+    for nic in nics:
+        if not isinstance(nic, collections.Mapping) or len(nic) != 1:
+            raise TypeError("Each NIC must be a dict with one item, "
+                            "got %s" % nic)
+
+        nic_type = next(iter(nic))
+        if nic_type not in ('port', 'network'):
+            unknown_nic_types.add(nic_type)
+
+    if unknown_nic_types:
+        raise ValueError("Unexpected NIC type(s) %s, supported values are "
+                         "'port' and 'network'" % ', '.join(unknown_nic_types))
