@@ -22,6 +22,8 @@ import six
 import testtools
 
 from metalsmith import _cmd
+from metalsmith import _config
+from metalsmith import _instance
 from metalsmith import _provisioner
 
 
@@ -37,14 +39,15 @@ class TestDeploy(testtools.TestCase):
     @mock.patch.object(_cmd, 'logging', autospec=True)
     def test_args_ok(self, mock_log, mock_os_conf, mock_pr):
         instance = mock_pr.return_value.provision_node.return_value
-        instance.create_autospec(_provisioner.Instance)
+        instance.create_autospec(_instance.Instance)
         instance.node.name = None
         instance.node.uuid = '123'
         instance.state = 'active'
         instance.is_deployed = True
         instance.ip_addresses.return_value = {'private': ['1.2.3.4']}
 
-        args = ['deploy', '--network', 'mynet', '--image', 'myimg', 'compute']
+        args = ['deploy', '--network', 'mynet', '--image', 'myimg',
+                '--resource-class', 'compute']
         _cmd.main(args)
 
         mock_pr.assert_called_once_with(
@@ -56,13 +59,15 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=[{'network': 'mynet'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=1800)
+        config = mock_pr.return_value.provision_node.call_args[1]['config']
+        self.assertEqual([], config.ssh_keys)
         mock_log.basicConfig.assert_called_once_with(level=mock_log.WARNING,
                                                      format=mock.ANY)
         self.assertEqual(
@@ -79,11 +84,11 @@ class TestDeploy(testtools.TestCase):
     @mock.patch.object(_cmd, 'logging', autospec=True)
     def test_args_json_format(self, mock_log, mock_os_conf, mock_pr):
         instance = mock_pr.return_value.provision_node.return_value
-        instance.create_autospec(_provisioner.Instance)
+        instance.create_autospec(_instance.Instance)
         instance.to_dict.return_value = {'node': 'dict'}
 
         args = ['--format', 'json', 'deploy', '--network', 'mynet',
-                '--image', 'myimg', 'compute']
+                '--image', 'myimg', '--resource-class', 'compute']
         fake_io = six.StringIO()
         with mock.patch('sys.stdout', fake_io):
             _cmd.main(args)
@@ -99,10 +104,10 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=[{'network': 'mynet'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=1800)
@@ -116,14 +121,15 @@ class TestDeploy(testtools.TestCase):
 
     def test_no_ips(self, mock_os_conf, mock_pr):
         instance = mock_pr.return_value.provision_node.return_value
-        instance.create_autospec(_provisioner.Instance)
+        instance.create_autospec(_instance.Instance)
         instance.is_deployed = True
         instance.ip_addresses.return_value = {}
         instance.node.name = None
         instance.node.uuid = '123'
         instance.state = 'active'
 
-        args = ['deploy', '--network', 'mynet', '--image', 'myimg', 'compute']
+        args = ['deploy', '--network', 'mynet', '--image', 'myimg',
+                '--resource-class', 'compute']
         _cmd.main(args)
 
         self.mock_print.assert_called_once_with(mock.ANY, node='123',
@@ -131,13 +137,14 @@ class TestDeploy(testtools.TestCase):
 
     def test_not_deployed_no_ips(self, mock_os_conf, mock_pr):
         instance = mock_pr.return_value.provision_node.return_value
-        instance.create_autospec(_provisioner.Instance)
+        instance.create_autospec(_instance.Instance)
         instance.is_deployed = False
         instance.node.name = None
         instance.node.uuid = '123'
         instance.state = 'deploying'
 
-        args = ['deploy', '--network', 'mynet', '--image', 'myimg', 'compute']
+        args = ['deploy', '--network', 'mynet', '--image', 'myimg',
+                '--resource-class', 'compute']
         _cmd.main(args)
 
         self.mock_print.assert_called_once_with(mock.ANY, node='123',
@@ -146,10 +153,11 @@ class TestDeploy(testtools.TestCase):
     @mock.patch.object(_cmd.LOG, 'info', autospec=True)
     def test_no_logs_not_deployed(self, mock_log, mock_os_conf, mock_pr):
         instance = mock_pr.return_value.provision_node.return_value
-        instance.create_autospec(_provisioner.Instance)
+        instance.create_autospec(_instance.Instance)
         instance.is_deployed = False
 
-        args = ['deploy', '--network', 'mynet', '--image', 'myimg', 'compute']
+        args = ['deploy', '--network', 'mynet', '--image', 'myimg',
+                '--resource-class', 'compute']
         _cmd.main(args)
 
         self.assertFalse(mock_log.called)
@@ -157,7 +165,7 @@ class TestDeploy(testtools.TestCase):
 
     def test_args_dry_run(self, mock_os_conf, mock_pr):
         args = ['--dry-run', 'deploy', '--network', 'mynet',
-                '--image', 'myimg', 'compute']
+                '--image', 'myimg', '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -168,10 +176,10 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=[{'network': 'mynet'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=1800)
@@ -179,7 +187,7 @@ class TestDeploy(testtools.TestCase):
     @mock.patch.object(_cmd, 'logging', autospec=True)
     def test_args_debug(self, mock_log, mock_os_conf, mock_pr):
         args = ['--debug', 'deploy', '--network', 'mynet', '--image', 'myimg',
-                'compute']
+                '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -190,10 +198,10 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=[{'network': 'mynet'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=1800)
@@ -209,7 +217,7 @@ class TestDeploy(testtools.TestCase):
     @mock.patch.object(_cmd, 'logging', autospec=True)
     def test_args_quiet(self, mock_log, mock_os_conf, mock_pr):
         args = ['--quiet', 'deploy', '--network', 'mynet', '--image', 'myimg',
-                'compute']
+                '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -220,10 +228,10 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=[{'network': 'mynet'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=1800)
@@ -241,7 +249,7 @@ class TestDeploy(testtools.TestCase):
     @mock.patch.object(_cmd, 'logging', autospec=True)
     def test_args_verbose_1(self, mock_log, mock_os_conf, mock_pr):
         args = ['-v', 'deploy', '--network', 'mynet', '--image', 'myimg',
-                'compute']
+                '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -252,10 +260,10 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=[{'network': 'mynet'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=1800)
@@ -271,7 +279,7 @@ class TestDeploy(testtools.TestCase):
     @mock.patch.object(_cmd, 'logging', autospec=True)
     def test_args_verbose_2(self, mock_log, mock_os_conf, mock_pr):
         args = ['-vv', 'deploy', '--network', 'mynet', '--image', 'myimg',
-                'compute']
+                '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -282,10 +290,10 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=[{'network': 'mynet'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=1800)
@@ -301,7 +309,7 @@ class TestDeploy(testtools.TestCase):
     @mock.patch.object(_cmd, 'logging', autospec=True)
     def test_args_verbose_3(self, mock_log, mock_os_conf, mock_pr):
         args = ['-vvv', 'deploy', '--network', 'mynet', '--image', 'myimg',
-                'compute']
+                '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -312,10 +320,10 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=[{'network': 'mynet'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=1800)
@@ -330,7 +338,8 @@ class TestDeploy(testtools.TestCase):
 
     @mock.patch.object(_cmd.LOG, 'critical', autospec=True)
     def test_reservation_failure(self, mock_log, mock_os_conf, mock_pr):
-        args = ['deploy', '--network', 'mynet', '--image', 'myimg', 'compute']
+        args = ['deploy', '--network', 'mynet', '--image', 'myimg',
+                '--resource-class', 'compute']
         failure = RuntimeError('boom')
         mock_pr.return_value.reserve_node.side_effect = failure
         self.assertRaises(SystemExit, _cmd.main, args)
@@ -338,7 +347,8 @@ class TestDeploy(testtools.TestCase):
 
     @mock.patch.object(_cmd.LOG, 'critical', autospec=True)
     def test_deploy_failure(self, mock_log, mock_os_conf, mock_pr):
-        args = ['deploy', '--network', 'mynet', '--image', 'myimg', 'compute']
+        args = ['deploy', '--network', 'mynet', '--image', 'myimg',
+                '--resource-class', 'compute']
         failure = RuntimeError('boom')
         mock_pr.return_value.provision_node.side_effect = failure
         self.assertRaises(SystemExit, _cmd.main, args)
@@ -346,14 +356,15 @@ class TestDeploy(testtools.TestCase):
 
     @mock.patch.object(_cmd.LOG, 'critical', autospec=True)
     def test_invalid_hostname(self, mock_log, mock_os_conf, mock_pr):
-        args = ['deploy', '--hostname', 'n_1', '--image', 'myimg', 'compute']
+        args = ['deploy', '--hostname', 'n_1', '--image', 'myimg',
+                '--resource-class', 'compute']
         self.assertRaises(SystemExit, _cmd.main, args)
         self.assertTrue(mock_log.called)
 
     def test_args_capabilities(self, mock_os_conf, mock_pr):
         args = ['deploy', '--network', 'mynet', '--image', 'myimg',
                 '--capability', 'foo=bar', '--capability', 'answer=42',
-                'compute']
+                '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -364,10 +375,10 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=[{'network': 'mynet'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=1800)
@@ -378,7 +389,7 @@ class TestDeploy(testtools.TestCase):
             fp.flush()
 
             args = ['deploy', '--network', 'mynet', '--image', 'myimg',
-                    '--ssh-public-key', fp.name, 'compute']
+                    '--ssh-public-key', fp.name, '--resource-class', 'compute']
             _cmd.main(args)
             mock_pr.assert_called_once_with(
                 cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -389,16 +400,20 @@ class TestDeploy(testtools.TestCase):
             )
             mock_pr.return_value.provision_node.assert_called_once_with(
                 mock_pr.return_value.reserve_node.return_value,
-                image_ref='myimg',
+                image='myimg',
                 nics=[{'network': 'mynet'}],
                 root_disk_size=None,
-                ssh_keys=['foo'],
+                config=mock.ANY,
                 hostname=None,
                 netboot=False,
                 wait=1800)
+        config = mock_pr.return_value.provision_node.call_args[1]['config']
+        self.assertEqual(['foo'], config.ssh_keys)
 
-    def test_args_port(self, mock_os_conf, mock_pr):
-        args = ['deploy', '--port', 'myport', '--image', 'myimg', 'compute']
+    @mock.patch.object(_config.InstanceConfig, 'add_user', autospec=True)
+    def test_args_user_name(self, mock_add_user, mock_os_conf, mock_pr):
+        args = ['deploy', '--network', 'mynet', '--image', 'myimg',
+                '--user-name', 'banana', '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -409,16 +424,67 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
+            nics=[{'network': 'mynet'}],
+            root_disk_size=None,
+            config=mock.ANY,
+            hostname=None,
+            netboot=False,
+            wait=1800)
+        config = mock_pr.return_value.provision_node.call_args[1]['config']
+        self.assertEqual([], config.ssh_keys)
+        mock_add_user.assert_called_once_with(config, 'banana', sudo=False)
+
+    @mock.patch.object(_config.InstanceConfig, 'add_user', autospec=True)
+    def test_args_user_name_with_sudo(self, mock_add_user, mock_os_conf,
+                                      mock_pr):
+        args = ['deploy', '--network', 'mynet', '--image', 'myimg',
+                '--user-name', 'banana', '--resource-class', 'compute',
+                '--passwordless-sudo']
+        _cmd.main(args)
+        mock_pr.assert_called_once_with(
+            cloud_region=mock_os_conf.return_value.get_one.return_value,
+            dry_run=False)
+        mock_pr.return_value.reserve_node.assert_called_once_with(
+            resource_class='compute',
+            capabilities={}
+        )
+        mock_pr.return_value.provision_node.assert_called_once_with(
+            mock_pr.return_value.reserve_node.return_value,
+            image='myimg',
+            nics=[{'network': 'mynet'}],
+            root_disk_size=None,
+            config=mock.ANY,
+            hostname=None,
+            netboot=False,
+            wait=1800)
+        config = mock_pr.return_value.provision_node.call_args[1]['config']
+        self.assertEqual([], config.ssh_keys)
+        mock_add_user.assert_called_once_with(config, 'banana', sudo=True)
+
+    def test_args_port(self, mock_os_conf, mock_pr):
+        args = ['deploy', '--port', 'myport', '--image', 'myimg',
+                '--resource-class', 'compute']
+        _cmd.main(args)
+        mock_pr.assert_called_once_with(
+            cloud_region=mock_os_conf.return_value.get_one.return_value,
+            dry_run=False)
+        mock_pr.return_value.reserve_node.assert_called_once_with(
+            resource_class='compute',
+            capabilities={}
+        )
+        mock_pr.return_value.provision_node.assert_called_once_with(
+            mock_pr.return_value.reserve_node.return_value,
+            image='myimg',
             nics=[{'port': 'myport'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=1800)
 
     def test_args_no_nics(self, mock_os_conf, mock_pr):
-        args = ['deploy', '--image', 'myimg', 'compute']
+        args = ['deploy', '--image', 'myimg', '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -429,10 +495,10 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=None,
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=1800)
@@ -440,7 +506,7 @@ class TestDeploy(testtools.TestCase):
     def test_args_networks_and_ports(self, mock_os_conf, mock_pr):
         args = ['deploy', '--network', 'net1', '--port', 'port1',
                 '--port', 'port2', '--network', 'net2',
-                '--image', 'myimg', 'compute']
+                '--image', 'myimg', '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -451,17 +517,18 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=[{'network': 'net1'}, {'port': 'port1'},
                   {'port': 'port2'}, {'network': 'net2'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=1800)
 
     def test_args_hostname(self, mock_os_conf, mock_pr):
-        args = ['deploy', '--hostname', 'host', '--image', 'myimg', 'compute']
+        args = ['deploy', '--hostname', 'host', '--image', 'myimg',
+                '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -472,17 +539,17 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=None,
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname='host',
             netboot=False,
             wait=1800)
 
     def test_args_custom_wait(self, mock_os_conf, mock_pr):
         args = ['deploy', '--network', 'mynet', '--image', 'myimg',
-                '--wait', '3600', 'compute']
+                '--wait', '3600', '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -493,17 +560,17 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=[{'network': 'mynet'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=3600)
 
     def test_args_no_wait(self, mock_os_conf, mock_pr):
         args = ['deploy', '--network', 'mynet', '--image', 'myimg',
-                '--no-wait', 'compute']
+                '--no-wait', '--resource-class', 'compute']
         _cmd.main(args)
         mock_pr.assert_called_once_with(
             cloud_region=mock_os_conf.return_value.get_one.return_value,
@@ -514,10 +581,10 @@ class TestDeploy(testtools.TestCase):
         )
         mock_pr.return_value.provision_node.assert_called_once_with(
             mock_pr.return_value.reserve_node.return_value,
-            image_ref='myimg',
+            image='myimg',
             nics=[{'network': 'mynet'}],
             root_disk_size=None,
-            ssh_keys=[],
+            config=mock.ANY,
             hostname=None,
             netboot=False,
             wait=None)
@@ -619,7 +686,7 @@ class TestShow(testtools.TestCase):
             'metalsmith._format._print', autospec=True))
         self.mock_print = self.print_fixture.mock
         self.instances = [
-            mock.Mock(spec=_provisioner.Instance, hostname=hostname,
+            mock.Mock(spec=_instance.Instance, hostname=hostname,
                       uuid=hostname[-1], is_deployed=(hostname[-1] == '1'),
                       state=('active' if hostname[-1] == '1' else 'deploying'),
                       **{'ip_addresses.return_value': {'private':
