@@ -13,9 +13,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import fixtures
 import mock
 import testtools
 
+from metalsmith import _config
+from metalsmith import _instance
 from metalsmith import _os_api
 from metalsmith import _provisioner
 from metalsmith import exceptions
@@ -96,7 +99,7 @@ class TestProvisionNode(Base):
 
     def setUp(self):
         super(TestProvisionNode, self).setUp()
-        image = self.api.get_image_info.return_value
+        image = self.api.get_image.return_value
         self.node.instance_uuid = self.node.uuid
         self.updates = {
             '/instance_info/ramdisk': image.ramdisk_id,
@@ -112,6 +115,10 @@ class TestProvisionNode(Base):
             ],
             '/instance_info/%s' % _os_api.HOSTNAME_FIELD: 'control-0'
         }
+        self.wait_fixture = self.useFixture(
+            fixtures.MockPatchObject(_provisioner.Provisioner,
+                                     '_wait_for_state', autospec=True))
+        self.wait_mock = self.wait_fixture.mock
 
     def test_ok(self):
         inst = self.pr.provision_node(self.node, 'image',
@@ -129,7 +136,31 @@ class TestProvisionNode(Base):
                                                        validate_deploy=True)
         self.api.node_action.assert_called_once_with(self.node, 'active',
                                                      configdrive=mock.ANY)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
+        self.assertFalse(self.api.release_node.called)
+        self.assertFalse(self.api.delete_port.called)
+
+    def test_with_config(self):
+        config = mock.MagicMock(spec=_config.InstanceConfig)
+        inst = self.pr.provision_node(self.node, 'image',
+                                      [{'network': 'network'}],
+                                      config=config)
+
+        self.assertEqual(inst.uuid, self.node.uuid)
+        self.assertEqual(inst.node, self.node)
+
+        config.build_configdrive_directory.assert_called_once_with(
+            self.node, self.node.name)
+        self.api.create_port.assert_called_once_with(
+            network_id=self.api.get_network.return_value.id)
+        self.api.attach_port_to_node.assert_called_once_with(
+            self.node.uuid, self.api.create_port.return_value.id)
+        self.api.update_node.assert_called_once_with(self.node, self.updates)
+        self.api.validate_node.assert_called_once_with(self.node,
+                                                       validate_deploy=True)
+        self.api.node_action.assert_called_once_with(self.node, 'active',
+                                                     configdrive=mock.ANY)
+        self.assertFalse(self.wait_mock.called)
         self.assertFalse(self.api.release_node.called)
         self.assertFalse(self.api.delete_port.called)
 
@@ -152,7 +183,7 @@ class TestProvisionNode(Base):
                                                        validate_deploy=True)
         self.api.node_action.assert_called_once_with(self.node, 'active',
                                                      configdrive=mock.ANY)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.assertFalse(self.api.release_node.called)
         self.assertFalse(self.api.delete_port.called)
 
@@ -174,7 +205,7 @@ class TestProvisionNode(Base):
                                                        validate_deploy=True)
         self.api.node_action.assert_called_once_with(self.node, 'active',
                                                      configdrive=mock.ANY)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.assertFalse(self.api.release_node.called)
         self.assertFalse(self.api.delete_port.called)
 
@@ -194,7 +225,7 @@ class TestProvisionNode(Base):
                                                        validate_deploy=True)
         self.api.node_action.assert_called_once_with(self.node, 'active',
                                                      configdrive=mock.ANY)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.assertFalse(self.api.release_node.called)
         self.assertFalse(self.api.delete_port.called)
 
@@ -218,12 +249,12 @@ class TestProvisionNode(Base):
                                                        validate_deploy=True)
         self.api.node_action.assert_called_once_with(self.node, 'active',
                                                      configdrive=mock.ANY)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.assertFalse(self.api.release_node.called)
         self.assertFalse(self.api.delete_port.called)
 
     def test_whole_disk(self):
-        image = self.api.get_image_info.return_value
+        image = self.api.get_image.return_value
         image.kernel_id = None
         image.ramdisk_id = None
         del self.updates['/instance_info/kernel']
@@ -240,7 +271,7 @@ class TestProvisionNode(Base):
                                                        validate_deploy=True)
         self.api.node_action.assert_called_once_with(self.node, 'active',
                                                      configdrive=mock.ANY)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.assertFalse(self.api.release_node.called)
         self.assertFalse(self.api.delete_port.called)
 
@@ -259,7 +290,7 @@ class TestProvisionNode(Base):
                                                        validate_deploy=True)
         self.api.node_action.assert_called_once_with(self.node, 'active',
                                                      configdrive=mock.ANY)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.assertFalse(self.api.release_node.called)
         self.assertFalse(self.api.delete_port.called)
 
@@ -280,9 +311,11 @@ class TestProvisionNode(Base):
                                                        validate_deploy=True)
         self.api.node_action.assert_called_once_with(self.node, 'active',
                                                      configdrive=mock.ANY)
-        self.api.wait_for_node_state.assert_called_once_with(self.node,
-                                                             'active',
-                                                             timeout=3600)
+        self.wait_mock.assert_called_once_with(self.pr,
+                                               [self.node],
+                                               'active',
+                                               delay=15,
+                                               timeout=3600)
         self.assertFalse(self.api.release_node.called)
         self.assertFalse(self.api.delete_port.called)
 
@@ -294,7 +327,7 @@ class TestProvisionNode(Base):
         self.assertFalse(self.api.attach_port_to_node.called)
         self.assertFalse(self.api.update_node.called)
         self.assertFalse(self.api.node_action.called)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.assertFalse(self.api.release_node.called)
         self.assertFalse(self.api.delete_port.called)
 
@@ -309,7 +342,7 @@ class TestProvisionNode(Base):
         self.assertFalse(self.api.attach_port_to_node.called)
         self.assertFalse(self.api.update_node.called)
         self.assertFalse(self.api.node_action.called)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.assertFalse(self.api.release_node.called)
         self.assertFalse(self.api.delete_port.called)
 
@@ -321,7 +354,7 @@ class TestProvisionNode(Base):
                                wait=3600)
 
         self.api.update_node.assert_any_call(self.node, CLEAN_UP)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.api.release_node.assert_called_once_with(self.node)
         self.api.delete_port.assert_called_once_with(
             self.api.create_port.return_value.id)
@@ -369,7 +402,7 @@ class TestProvisionNode(Base):
                                    'image', [{'network': 'network'}],
                                    wait=3600)
 
-            self.assertFalse(self.api.wait_for_node_state.called)
+            self.assertFalse(self.wait_mock.called)
             self.api.release_node.assert_called_once_with(self.node)
             self.api.delete_port.assert_called_once_with(
                 self.api.create_port.return_value.id)
@@ -386,7 +419,7 @@ class TestProvisionNode(Base):
                                'image', [{'network': 'network'}],
                                wait=3600)
 
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.api.release_node.assert_called_once_with(self.node)
         self.api.delete_port.assert_called_once_with(
             self.api.create_port.return_value.id)
@@ -394,7 +427,7 @@ class TestProvisionNode(Base):
             self.node, self.api.create_port.return_value.id)
 
     def test_wait_failure(self):
-        self.api.wait_for_node_state.side_effect = RuntimeError('boom')
+        self.wait_mock.side_effect = RuntimeError('boom')
         self.assertRaisesRegex(RuntimeError, 'boom',
                                self.pr.provision_node, self.node,
                                'image', [{'network': 'network'}], wait=3600)
@@ -407,7 +440,7 @@ class TestProvisionNode(Base):
         self.assertFalse(self.api.detach_port_from_node.called)
 
     def test_missing_image(self):
-        self.api.get_image_info.side_effect = RuntimeError('Not found')
+        self.api.get_image.side_effect = RuntimeError('Not found')
         self.assertRaisesRegex(exceptions.InvalidImage, 'Not found',
                                self.pr.provision_node,
                                self.node, 'image', [{'network': 'network'}])
@@ -487,7 +520,7 @@ class TestProvisionNode(Base):
         self.api.release_node.assert_called_with(self.node)
 
     def test_invalid_nic_type(self):
-        self.assertRaisesRegex(ValueError, 'Unexpected NIC type foo',
+        self.assertRaisesRegex(ValueError, r'Unexpected NIC type\(s\) foo',
                                self.pr.provision_node,
                                self.node, 'image', [{'foo': 'bar'}])
         self.assertFalse(self.api.create_port.called)
@@ -553,6 +586,13 @@ class TestProvisionNode(Base):
 
 class TestUnprovisionNode(Base):
 
+    def setUp(self):
+        super(TestUnprovisionNode, self).setUp()
+        self.wait_fixture = self.useFixture(
+            fixtures.MockPatchObject(_provisioner.Provisioner,
+                                     '_wait_for_state', autospec=True))
+        self.wait_mock = self.wait_fixture.mock
+
     def test_ok(self):
         self.node.extra['metalsmith_created_ports'] = ['port1']
         result = self.pr.unprovision_node(self.node)
@@ -563,7 +603,7 @@ class TestUnprovisionNode(Base):
                                                                'port1')
         self.api.node_action.assert_called_once_with(self.node, 'deleted')
         self.api.release_node.assert_called_once_with(self.node)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.api.update_node.assert_called_once_with(self.node, CLEAN_UP)
 
     def test_with_attached(self):
@@ -576,7 +616,7 @@ class TestUnprovisionNode(Base):
         self.api.detach_port_from_node.assert_has_calls(calls, any_order=True)
         self.api.node_action.assert_called_once_with(self.node, 'deleted')
         self.api.release_node.assert_called_once_with(self.node)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.api.update_node.assert_called_once_with(self.node, CLEAN_UP)
 
     def test_with_wait(self):
@@ -589,9 +629,10 @@ class TestUnprovisionNode(Base):
                                                                'port1')
         self.api.node_action.assert_called_once_with(self.node, 'deleted')
         self.api.release_node.assert_called_once_with(self.node)
-        self.api.wait_for_node_state.assert_called_once_with(self.node,
-                                                             'available',
-                                                             timeout=3600)
+        self.wait_mock.assert_called_once_with(self.pr,
+                                               [self.node],
+                                               'available',
+                                               timeout=3600)
 
     def test_dry_run(self):
         self.pr._dry_run = True
@@ -602,7 +643,7 @@ class TestUnprovisionNode(Base):
         self.assertFalse(self.api.release_node.called)
         self.assertFalse(self.api.delete_port.called)
         self.assertFalse(self.api.detach_port_from_node.called)
-        self.assertFalse(self.api.wait_for_node_state.called)
+        self.assertFalse(self.wait_mock.called)
         self.assertFalse(self.api.update_node.called)
 
 
@@ -612,7 +653,7 @@ class TestShowInstance(Base):
         inst = self.pr.show_instance('uuid1')
         self.api.get_node.assert_called_once_with('uuid1',
                                                   accept_hostname=True)
-        self.assertIsInstance(inst, _provisioner.Instance)
+        self.assertIsInstance(inst, _instance.Instance)
         self.assertIs(inst.node, self.node)
         self.assertIs(inst.uuid, self.node.uuid)
         self.api.cache_node_list_for_lookup.assert_called_once_with()
@@ -626,104 +667,98 @@ class TestShowInstance(Base):
         ])
         self.assertIsInstance(result, list)
         for inst in result:
-            self.assertIsInstance(inst, _provisioner.Instance)
+            self.assertIsInstance(inst, _instance.Instance)
         self.assertIs(result[0].node, self.node)
         self.assertIs(result[0].uuid, self.node.uuid)
         self.api.cache_node_list_for_lookup.assert_called_once_with()
 
 
-class TestInstanceStates(Base):
-    def setUp(self):
-        super(TestInstanceStates, self).setUp()
-        self.instance = _provisioner.Instance(self.api, self.node)
+@mock.patch('time.sleep', autospec=True)
+class TestWaitForState(Base):
+    def test_invalid_timeout(self, mock_sleep):
+        for invalid in (0, -42):
+            self.assertRaisesRegex(ValueError,
+                                   'timeout argument must be a positive',
+                                   self.pr.wait_for_provisioning,
+                                   ['uuid1'], timeout=invalid)
 
-    def test_state_deploying(self):
-        self.node.provision_state = 'wait call-back'
-        self.assertEqual('deploying', self.instance.state)
-        self.assertFalse(self.instance.is_deployed)
-        self.assertTrue(self.instance.is_healthy)
+    def test_invalid_delay(self, mock_sleep):
+        self.assertRaisesRegex(ValueError,
+                               'delay argument must be a non-negative',
+                               self.pr.wait_for_provisioning,
+                               ['uuid1'], delay=-42)
 
-    def test_state_deploying_when_available(self):
-        self.node.provision_state = 'available'
-        self.assertEqual('deploying', self.instance.state)
-        self.assertFalse(self.instance.is_deployed)
-        self.assertTrue(self.instance.is_healthy)
-
-    def test_state_deploying_maintenance(self):
-        self.node.maintenance = True
-        self.node.provision_state = 'wait call-back'
-        self.assertEqual('deploying', self.instance.state)
-        self.assertFalse(self.instance.is_deployed)
-        self.assertFalse(self.instance.is_healthy)
-
-    def test_state_active(self):
-        self.node.provision_state = 'active'
-        self.assertEqual('active', self.instance.state)
-        self.assertTrue(self.instance.is_deployed)
-        self.assertTrue(self.instance.is_healthy)
-
-    def test_state_maintenance(self):
-        self.node.maintenance = True
-        self.node.provision_state = 'active'
-        self.assertEqual('maintenance', self.instance.state)
-        self.assertTrue(self.instance.is_deployed)
-        self.assertFalse(self.instance.is_healthy)
-
-    def test_state_error(self):
-        self.node.provision_state = 'deploy failed'
-        self.assertEqual('error', self.instance.state)
-        self.assertFalse(self.instance.is_deployed)
-        self.assertFalse(self.instance.is_healthy)
-
-    def test_state_unknown(self):
-        self.node.provision_state = 'enroll'
-        self.assertEqual('unknown', self.instance.state)
-        self.assertFalse(self.instance.is_deployed)
-        self.assertFalse(self.instance.is_healthy)
-
-    @mock.patch.object(_provisioner.Instance, 'ip_addresses', autospec=True)
-    def test_to_dict(self, mock_ips):
-        self.node.provision_state = 'wait call-back'
-        self.node.to_dict.return_value = {'node': 'dict'}
-        self.node.instance_info = {'metalsmith_hostname': 'host'}
-        mock_ips.return_value = {'private': ['1.2.3.4']}
-
-        self.assertEqual({'hostname': 'host',
-                          'ip_addresses': {'private': ['1.2.3.4']},
-                          'node': {'node': 'dict'},
-                          'state': 'deploying',
-                          'uuid': self.node.uuid},
-                         self.instance.to_dict())
-
-
-class TestInstanceIPAddresses(Base):
-    def setUp(self):
-        super(TestInstanceIPAddresses, self).setUp()
-        self.instance = _provisioner.Instance(self.api, self.node)
-        self.api.list_node_attached_ports.return_value = [
-            mock.Mock(spec=['id'], id=i) for i in ('111', '222')
+    def test_success_one_node(self, mock_sleep):
+        nodes = [
+            mock.Mock(spec=_os_api.NODE_FIELDS, provision_state=state)
+            for state in ('deploying', 'deploy wait', 'deploying', 'active')
         ]
-        self.ports = [
-            mock.Mock(spec=['network_id', 'fixed_ips', 'network'],
-                      network_id=n, fixed_ips=[{'ip_address': ip}])
-            for n, ip in [('0', '192.168.0.1'), ('1', '10.0.0.2')]
-        ]
-        self.api.get_port.side_effect = self.ports
-        self.nets = [
-            mock.Mock(spec=['id', 'name'], id=str(i)) for i in range(2)
-        ]
-        for n in self.nets:
-            n.name = 'name-%s' % n.id
-        self.api.get_network.side_effect = self.nets
+        self.api.get_node.side_effect = nodes
 
-    def test_ip_addresses(self):
-        ips = self.instance.ip_addresses()
-        self.assertEqual({'name-0': ['192.168.0.1'],
-                          'name-1': ['10.0.0.2']},
-                         ips)
+        result = self.pr.wait_for_provisioning(['uuid1'])
+        self.assertEqual(nodes[-1:], result)
 
-    def test_missing_ip(self):
-        self.ports[0].fixed_ips = {}
-        ips = self.instance.ip_addresses()
-        self.assertEqual({'name-0': [],
-                          'name-1': ['10.0.0.2']}, ips)
+        mock_sleep.assert_called_with(15)
+        self.assertEqual(3, mock_sleep.call_count)
+
+    def test_success_several_nodes(self, mock_sleep):
+        nodes = [
+            mock.Mock(spec=_os_api.NODE_FIELDS, provision_state=state)
+            for state in ('deploying', 'deploy wait',  # iteration 1
+                          'deploying', 'active',       # iteration 2
+                          'active')                    # iteration 3
+        ]
+        self.api.get_node.side_effect = nodes
+
+        result = self.pr.wait_for_provisioning(['uuid1', 'uuid2'])
+        self.assertEqual(nodes[-2:], result)
+
+        mock_sleep.assert_called_with(15)
+        self.assertEqual(2, mock_sleep.call_count)
+
+    def test_one_node_failed(self, mock_sleep):
+        nodes = [
+            mock.Mock(spec=_os_api.NODE_FIELDS, provision_state=state)
+            for state in ('deploying', 'deploy wait',    # iteration 1
+                          'deploying', 'deploy failed',  # iteration 2
+                          'active')                      # iteration 3
+        ]
+        self.api.get_node.side_effect = nodes
+
+        exc = self.assertRaises(exceptions.DeploymentFailure,
+                                self.pr.wait_for_provisioning,
+                                ['uuid1', 'uuid2'])
+        # The exception contains the failed node
+        self.assertEqual(exc.nodes, [nodes[-2]])
+
+        mock_sleep.assert_called_with(15)
+        self.assertEqual(2, mock_sleep.call_count)
+
+    def test_timeout(self, mock_sleep):
+        def _fake_get(*args, **kwargs):
+            while True:
+                yield mock.Mock(spec=_os_api.NODE_FIELDS,
+                                provision_state='deploying')
+
+        self.api.get_node.side_effect = _fake_get()
+
+        exc = self.assertRaises(exceptions.DeploymentFailure,
+                                self.pr.wait_for_provisioning,
+                                ['uuid1', 'uuid2'],
+                                timeout=0.001)
+        self.assertEqual(2, len(exc.nodes))
+
+        mock_sleep.assert_called_with(15)
+
+    def test_custom_delay(self, mock_sleep):
+        nodes = [
+            mock.Mock(spec=_os_api.NODE_FIELDS, provision_state=state)
+            for state in ('deploying', 'deploy wait', 'deploying', 'active')
+        ]
+        self.api.get_node.side_effect = nodes
+
+        result = self.pr.wait_for_provisioning(['uuid1'], delay=1)
+        self.assertEqual(nodes[-1:], result)
+
+        mock_sleep.assert_called_with(1)
+        self.assertEqual(3, mock_sleep.call_count)
